# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import pandas as pd, numpy as np
from dataiku import pandasutils as pdu
from datetime import datetime, timedelta
import datetime as dt
import yfinance as yf
import requests
import functools

session = requests.Session()
# accept gzip compressed data for less bandwidth
session.headers.update({
    'Accept-Encoding': 'gzip, deflate',
})
session.request = functools.partial(session.request, timeout=30)
# replace requests library in yfinance
yf.base._requests = session
yf.utils._requests = session
yf.ticker._requests = session

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Read recipe inputs
stock_tickers = dataiku.Dataset("tickers_list")
stock_tickers_df = stock_tickers.get_dataframe()

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Compute recipe outputs from inputs
date_to_compute = dataiku.dku_flow_variables["DKU_DST_DATE"]
next_date = (dt.datetime.strptime(date_to_compute, "%Y-%m-%d") + dt.timedelta(days=1)).strftime("%Y-%m-%d")

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
symbols = list(stock_tickers_df.ticker)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
stock_prices = pd.DataFrame()

multitask_limit = 50

for i in range(len(symbols)//multitask_limit+1):
    stock_prices_temp = yf.download(symbols[(i*multitask_limit):min(len(symbols), (i+1)*multitask_limit)], start=date_to_compute, end=next_date)
    melt_stock_df = pd.melt(stock_prices_temp)
    melt_stock_df = melt_stock_df[melt_stock_df["variable_0"]=="Adj Close"]
    melt_stock_df = melt_stock_df.drop("variable_0", axis=1).dropna()
    melt_stock_df["date"] = date_to_compute
    stock_prices = stock_prices.append(melt_stock_df)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Write recipe outputs
stock_prices_dataset = dataiku.Dataset("stock_prices")
stock_prices_dataset.write_with_schema(stock_prices)